import { ipcMain, app } from 'electron'
import { MaProduct, MaRelease } from 'solar-js-sdk'
import { join } from 'path'
import * as fs from 'fs'
import { spawn } from 'child_process'

export interface AppLibrary {
  apps: LocalAppRecord[]
}

export let library: AppLibrary

function readLocalLibrary(): AppLibrary {
  const basePath = app.getPath('userData')
  const libraryPath = join(basePath, 'apps_library.json')
  console.log(`[Library] Loading library from ${libraryPath}`)
  if (fs.existsSync(libraryPath)) {
    const data = fs.readFileSync(libraryPath, 'utf-8')
    library = JSON.parse(data)
  } else {
    library = { apps: [] }
    saveLocalLibrary()
  }
  return library
}

function saveLocalLibrary() {
  const basePath = app.getPath('userData')
  const libraryPath = join(basePath, 'apps_library.json')
  fs.writeFileSync(libraryPath, JSON.stringify(library), 'utf-8')
}

export interface LocalAppRecord {
  id: string
  product: MaProduct
  release: MaRelease
  basePath: string
}

export function initLibrary(): void {
  readLocalLibrary()
  ipcMain.handle('get-app-library', () => JSON.stringify(getAppLibrary()))
  ipcMain.handle('get-app-library-one', (_, id: string) =>
    JSON.stringify(getAppLibrary().filter((ele) => ele.id === id)[0]),
  )

  ipcMain.handle('launch-app', (_, id: string) => launchApp(id))
}

export function getAppLibrary(): LocalAppRecord[] {
  return library.apps
}

export function setAppRecord(record: LocalAppRecord) {
  let current = getAppLibrary()
  let hit = false

  current = current.map((rec) => {
    if (rec.id === record.id) hit = true
    return rec.id === record.id ? record : rec
  })

  if (!hit) {
    current.push(record)
  }

  library.apps = current
  saveLocalLibrary()
}

export function launchApp(id: string): void {
  const app = getAppLibrary().filter((ele) => ele.id === id)[0]
  if (!app) return

  const platform = process.platform
  const runner = app.release.runners[platform]

  const segments = runner.script.split(' ')

  try {
    spawn(segments[0], segments.slice(1), {
      detached: true,
      cwd: runner.workdir ? join(app.basePath, runner.workdir) : app.basePath,
    })
  } catch (err: any) {
    console.error(err)
  }
}
