import { LocalAppRecord } from '@main/library'
import { Avatar, Box, Button, Container, Divider, Grid2 as Grid, IconButton, Stack, Typography } from '@mui/material'
import { useEffect, useMemo, useState } from 'react'
import { useNavigate, useParams } from 'react-router'
import { getAttachmentUrl } from 'solar-js-sdk'
import { parseContent } from '@renderer/services/parser'

import PlayIcon from '@mui/icons-material/PlayArrow'
import ArrowBackwardIcon from '@mui/icons-material/ArrowBack'

export default function LibraryDetails(): JSX.Element {
  const { id } = useParams()
  const navigate = useNavigate()

  const [app, setApp] = useState<LocalAppRecord>()
  const [appProductContent, setAppProductContent] = useState<string>()
  const [appReleaseContent, setAppReleaseContent] = useState<string>()

  const product = useMemo(() => app?.product, [app])

  function fetchApp() {
    window.electron.ipcRenderer.invoke('get-app-library-one', id).then((res) => {
      const app = JSON.parse(res)
      if (app == null) {
        navigate('/library')
        return
      }
      return setApp(app)
    })
  }

  useEffect(() => {
    fetchApp()
  }, [])

  useEffect(() => {
    if (app) {
      parseContent(app.product.meta.introduction).then((out) => setAppProductContent(out))
      parseContent(app.release.meta.content).then((out) => setAppReleaseContent(out))
    }
  }, [app])

  function launchApp() {
    window.electron.ipcRenderer.invoke('launch-app', app?.id)
  }

  return (
    <>
      {product?.previews && (
        <img
          src={product.previews[0]}
          style={{ aspectRatio: 16 / 5, width: '100%', objectFit: 'cover' }}
          className="border-b border-1 pb"
        />
      )}

      <Container sx={{ py: 4 }}>
        <Grid container spacing={2}>
          <Grid size={8} sx={{ display: 'flex', flexDirection: 'column', gap: 2 }}>
            <Box>
              <Button
                size="small"
                startIcon={<ArrowBackwardIcon />}
                onClick={() => navigate('/library')}
                sx={{ mb: 1 }}
              >
                Back
              </Button>
            </Box>

            <Box>
              {product?.icon && (
                <Avatar
                  variant="rounded"
                  src={getAttachmentUrl(product.icon)}
                  sx={{ width: 64, height: 64, border: 1, borderColor: 'divider', borderRadius: 4, mb: 1, mx: '-4px' }}
                />
              )}
              <Typography variant="h5" component="h1" maxWidth="sm">
                {product?.name}
              </Typography>
              <Typography variant="body1" component="p" maxWidth="sm">
                {product?.description}
              </Typography>
            </Box>

            <Divider sx={{ mb: 2 }} />

            <Typography variant="h6" component="h2" gutterBottom>
              Release Information
            </Typography>

            {appReleaseContent && (
              <Box
                component="article"
                className="prose prose-lg dark:prose-invert"
                dangerouslySetInnerHTML={{ __html: appReleaseContent ?? '' }}
              />
            )}

            <Divider sx={{ mb: 2 }} />

            <Typography variant="h6" component="h2" gutterBottom>
              Product Introduction
            </Typography>

            {appProductContent && (
              <Box
                component="article"
                className="prose prose-lg dark:prose-invert"
                dangerouslySetInnerHTML={{ __html: appProductContent ?? '' }}
              />
            )}
          </Grid>

          <Grid size={4}>
            <Stack spacing={2} alignItems="end">
              <Box>
                <IconButton color="success" onClick={launchApp}>
                  <PlayIcon />
                </IconButton>
              </Box>
            </Stack>
          </Grid>
        </Grid>
      </Container>
    </>
  )
}
